package org.apache.bcel.verifier.statics;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Locale;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import org.apache.bcel.Constants;
import org.apache.bcel.Repository;
import org.apache.bcel.classfile.Attribute;
import org.apache.bcel.classfile.ClassFormatException;
import org.apache.bcel.classfile.Code;
import org.apache.bcel.classfile.CodeException;
import org.apache.bcel.classfile.Constant;
import org.apache.bcel.classfile.ConstantClass;
import org.apache.bcel.classfile.ConstantDouble;
import org.apache.bcel.classfile.ConstantFieldref;
import org.apache.bcel.classfile.ConstantFloat;
import org.apache.bcel.classfile.ConstantInteger;
import org.apache.bcel.classfile.ConstantInterfaceMethodref;
import org.apache.bcel.classfile.ConstantLong;
import org.apache.bcel.classfile.ConstantMethodref;
import org.apache.bcel.classfile.ConstantNameAndType;
import org.apache.bcel.classfile.ConstantPool;
import org.apache.bcel.classfile.ConstantString;
import org.apache.bcel.classfile.ConstantUtf8;
import org.apache.bcel.classfile.ConstantValue;
import org.apache.bcel.classfile.Deprecated;
import org.apache.bcel.classfile.DescendingVisitor;
import org.apache.bcel.classfile.EmptyVisitor;
import org.apache.bcel.classfile.ExceptionTable;
import org.apache.bcel.classfile.Field;
import org.apache.bcel.classfile.InnerClass;
import org.apache.bcel.classfile.InnerClasses;
import org.apache.bcel.classfile.JavaClass;
import org.apache.bcel.classfile.LineNumber;
import org.apache.bcel.classfile.LineNumberTable;
import org.apache.bcel.classfile.LocalVariable;
import org.apache.bcel.classfile.LocalVariableTable;
import org.apache.bcel.classfile.Method;
import org.apache.bcel.classfile.Node;
import org.apache.bcel.classfile.SourceFile;
import org.apache.bcel.classfile.Synthetic;
import org.apache.bcel.classfile.Unknown;
import org.apache.bcel.classfile.Utility;
import org.apache.bcel.generic.ArrayType;
import org.apache.bcel.generic.ObjectType;
import org.apache.bcel.generic.Type;
import org.apache.bcel.verifier.PassVerifier;
import org.apache.bcel.verifier.VerificationResult;
import org.apache.bcel.verifier.Verifier;
import org.apache.bcel.verifier.VerifierFactory;
import org.apache.bcel.verifier.exc.AssertionViolatedException;
import org.apache.bcel.verifier.exc.ClassConstraintException;
import org.apache.bcel.verifier.exc.LocalVariableInfoInconsistentException;

public final class Pass2Verifier extends PassVerifier implements Constants {
   private LocalVariablesInfo[] localVariablesInfos;
   private final Verifier myOwner;

   private static String tostring(Node n) {
      return (new StringRepresentation(n)).toString();
   }

   private static boolean validClassMethodName(String name) {
      return validMethodName(name, false);
   }

   private static boolean validClassName(String name) {
      Objects.requireNonNull(name, "name");
      return true;
   }

   private static boolean validFieldName(String name) {
      return validJavaIdentifier(name);
   }

   private static boolean validInterfaceMethodName(String name) {
      return name.startsWith("<") ? false : validJavaLangMethodName(name);
   }

   private static boolean validJavaIdentifier(String name) {
      if (!name.isEmpty() && Character.isJavaIdentifierStart(name.charAt(0))) {
         for(int i = 1; i < name.length(); ++i) {
            if (!Character.isJavaIdentifierPart(name.charAt(i))) {
               return false;
            }
         }

         return true;
      } else {
         return false;
      }
   }

   private static boolean validJavaLangMethodName(String name) {
      return validJavaIdentifier(name);
   }

   private static boolean validMethodName(String name, boolean allowStaticInit) {
      if (validJavaLangMethodName(name)) {
         return true;
      } else if (!allowStaticInit) {
         return name.equals("<init>");
      } else {
         return name.equals("<init>") || name.equals("<clinit>");
      }
   }

   public Pass2Verifier(Verifier owner) {
      this.myOwner = owner;
   }

   private void constantPoolEntriesSatisfyStaticConstraints() {
      try {
         JavaClass jc = Repository.lookupClass(this.myOwner.getClassName());
         new CPESSC_Visitor(jc);
      } catch (ClassNotFoundException var2) {
         ClassNotFoundException e = var2;
         throw new AssertionViolatedException("Missing class: " + e, e);
      }
   }

   public VerificationResult do_verify() {
      try {
         VerificationResult vr1 = this.myOwner.doPass1();
         if (vr1.equals(VerificationResult.VR_OK)) {
            this.localVariablesInfos = new LocalVariablesInfo[Repository.lookupClass(this.myOwner.getClassName()).getMethods().length];
            VerificationResult vr = VerificationResult.VR_OK;

            try {
               this.constantPoolEntriesSatisfyStaticConstraints();
               this.fieldAndMethodRefsAreValid();
               this.everyClassHasAnAccessibleSuperclass();
               this.finalMethodsAreNotOverridden();
            } catch (ClassConstraintException var4) {
               ClassConstraintException cce = var4;
               vr = new VerificationResult(2, cce.getMessage());
            }

            return vr;
         } else {
            return VerificationResult.VR_NOTYET;
         }
      } catch (ClassNotFoundException var5) {
         ClassNotFoundException e = var5;
         throw new AssertionViolatedException("Missing class: " + e, e);
      }
   }

   private void everyClassHasAnAccessibleSuperclass() {
      try {
         Set<String> hs = new HashSet();
         JavaClass jc = Repository.lookupClass(this.myOwner.getClassName());
         int supidx = -1;

         while(supidx != 0) {
            supidx = jc.getSuperclassNameIndex();
            if (supidx == 0) {
               if (jc != Repository.lookupClass(Type.OBJECT.getClassName())) {
                  throw new ClassConstraintException("Superclass of '" + jc.getClassName() + "' missing but not " + Type.OBJECT.getClassName() + " itself!");
               }
            } else {
               String supername = jc.getSuperclassName();
               if (!hs.add(supername)) {
                  throw new ClassConstraintException("Circular superclass hierarchy detected.");
               }

               Verifier v = VerifierFactory.getVerifier(supername);
               VerificationResult vr = v.doPass1();
               if (vr != VerificationResult.VR_OK) {
                  throw new ClassConstraintException("Could not load in ancestor class '" + supername + "'.");
               }

               jc = Repository.lookupClass(supername);
               if (jc.isFinal()) {
                  throw new ClassConstraintException("Ancestor class '" + supername + "' has the FINAL access modifier and must therefore not be subclassed.");
               }
            }
         }

      } catch (ClassNotFoundException var7) {
         ClassNotFoundException e = var7;
         throw new AssertionViolatedException("Missing class: " + e, e);
      }
   }

   private void fieldAndMethodRefsAreValid() {
      try {
         JavaClass jc = Repository.lookupClass(this.myOwner.getClassName());
         DescendingVisitor v = new DescendingVisitor(jc, new FAMRAV_Visitor(jc));
         v.visit();
      } catch (ClassNotFoundException var3) {
         ClassNotFoundException e = var3;
         throw new AssertionViolatedException("Missing class: " + e, e);
      }
   }

   private void finalMethodsAreNotOverridden() {
      try {
         Map<String, String> map = new HashMap();
         JavaClass jc = Repository.lookupClass(this.myOwner.getClassName());

         for(int supidx = -1; supidx != 0; jc = Repository.lookupClass(jc.getSuperclassName())) {
            supidx = jc.getSuperclassNameIndex();
            Method[] methods = jc.getMethods();
            Method[] var5 = methods;
            int var6 = methods.length;

            for(int var7 = 0; var7 < var6; ++var7) {
               Method method = var5[var7];
               String nameAndSig = method.getName() + method.getSignature();
               if (map.containsKey(nameAndSig)) {
                  if (method.isFinal()) {
                     if (!method.isPrivate()) {
                        throw new ClassConstraintException("Method '" + nameAndSig + "' in class '" + (String)map.get(nameAndSig) + "' overrides the final (not-overridable) definition in class '" + jc.getClassName() + "'.");
                     }

                     this.addMessage("Method '" + nameAndSig + "' in class '" + (String)map.get(nameAndSig) + "' overrides the final (not-overridable) definition in class '" + jc.getClassName() + "'. This is okay, as the original definition was private; however this constraint leverage was introduced by JLS 8.4.6 (not vmspec2) and the behavior of the Sun verifiers.");
                  } else if (!method.isStatic()) {
                     map.put(nameAndSig, jc.getClassName());
                  }
               } else if (!method.isStatic()) {
                  map.put(nameAndSig, jc.getClassName());
               }
            }
         }

      } catch (ClassNotFoundException var10) {
         ClassNotFoundException e = var10;
         throw new AssertionViolatedException("Missing class: " + e, e);
      }
   }

   public LocalVariablesInfo getLocalVariablesInfo(int methodNr) {
      if (this.verify() != VerificationResult.VR_OK) {
         return null;
      } else if (methodNr >= 0 && methodNr < this.localVariablesInfos.length) {
         return this.localVariablesInfos[methodNr];
      } else {
         throw new AssertionViolatedException("Method number out of range.");
      }
   }

   private static class InnerClassDetector extends EmptyVisitor {
      private boolean hasInnerClass;
      private final JavaClass jc;
      private final ConstantPool cp;

      public InnerClassDetector(JavaClass javaClass) {
         this.jc = javaClass;
         this.cp = this.jc.getConstantPool();
         (new DescendingVisitor(this.jc, this)).visit();
      }

      public boolean innerClassReferenced() {
         return this.hasInnerClass;
      }

      public void visitConstantClass(ConstantClass obj) {
         Constant c = this.cp.getConstant(obj.getNameIndex());
         if (c instanceof ConstantUtf8) {
            String className = ((ConstantUtf8)c).getBytes();
            if (className.startsWith(this.jc.getClassName().replace('.', '/') + "$")) {
               this.hasInnerClass = true;
            }
         }

      }
   }

   private final class FAMRAV_Visitor extends EmptyVisitor {
      private final ConstantPool cp;

      private FAMRAV_Visitor(JavaClass jc) {
         this.cp = jc.getConstantPool();
      }

      public void visitConstantFieldref(ConstantFieldref obj) {
         if (obj.getTag() != 9) {
            throw new ClassConstraintException("ConstantFieldref '" + Pass2Verifier.tostring(obj) + "' has wrong tag!");
         } else {
            int nameAndTypeIndex = obj.getNameAndTypeIndex();
            ConstantNameAndType cnat = (ConstantNameAndType)this.cp.getConstant(nameAndTypeIndex);
            String name = ((ConstantUtf8)this.cp.getConstant(cnat.getNameIndex())).getBytes();
            if (!Pass2Verifier.validFieldName(name)) {
               throw new ClassConstraintException("Invalid field name '" + name + "' referenced by '" + Pass2Verifier.tostring(obj) + "'.");
            } else {
               int classIndex = obj.getClassIndex();
               ConstantClass cc = (ConstantClass)this.cp.getConstant(classIndex);
               String className = ((ConstantUtf8)this.cp.getConstant(cc.getNameIndex())).getBytes();
               if (!Pass2Verifier.validClassName(className)) {
                  throw new ClassConstraintException("Illegal class name '" + className + "' used by '" + Pass2Verifier.tostring(obj) + "'.");
               } else {
                  String sig = ((ConstantUtf8)this.cp.getConstant(cnat.getSignatureIndex())).getBytes();

                  try {
                     Type.getType(sig);
                  } catch (ClassFormatException var10) {
                     ClassFormatException cfe = var10;
                     throw new ClassConstraintException("Illegal descriptor (==signature) '" + sig + "' used by '" + Pass2Verifier.tostring(obj) + "'.", cfe);
                  }
               }
            }
         }
      }

      public void visitConstantInterfaceMethodref(ConstantInterfaceMethodref obj) {
         if (obj.getTag() != 11) {
            throw new ClassConstraintException("ConstantInterfaceMethodref '" + Pass2Verifier.tostring(obj) + "' has wrong tag!");
         } else {
            int nameAndTypeIndex = obj.getNameAndTypeIndex();
            ConstantNameAndType cnat = (ConstantNameAndType)this.cp.getConstant(nameAndTypeIndex);
            String name = ((ConstantUtf8)this.cp.getConstant(cnat.getNameIndex())).getBytes();
            if (!Pass2Verifier.validInterfaceMethodName(name)) {
               throw new ClassConstraintException("Invalid (interface) method name '" + name + "' referenced by '" + Pass2Verifier.tostring(obj) + "'.");
            } else {
               int classIndex = obj.getClassIndex();
               ConstantClass cc = (ConstantClass)this.cp.getConstant(classIndex);
               String className = ((ConstantUtf8)this.cp.getConstant(cc.getNameIndex())).getBytes();
               if (!Pass2Verifier.validClassName(className)) {
                  throw new ClassConstraintException("Illegal class name '" + className + "' used by '" + Pass2Verifier.tostring(obj) + "'.");
               } else {
                  String sig = ((ConstantUtf8)this.cp.getConstant(cnat.getSignatureIndex())).getBytes();

                  try {
                     Type t = Type.getReturnType(sig);
                     if (name.equals("<clinit>") && t != Type.VOID) {
                        Pass2Verifier.this.addMessage("Class or interface initialization method '<clinit>' usually has VOID return type instead of '" + t + "'. Note this is really not a requirement of The Java Virtual Machine Specification, Second Edition.");
                     }

                  } catch (ClassFormatException var10) {
                     ClassFormatException cfe = var10;
                     throw new ClassConstraintException("Illegal descriptor (==signature) '" + sig + "' used by '" + Pass2Verifier.tostring(obj) + "'.", cfe);
                  }
               }
            }
         }
      }

      public void visitConstantMethodref(ConstantMethodref obj) {
         if (obj.getTag() != 10) {
            throw new ClassConstraintException("ConstantMethodref '" + Pass2Verifier.tostring(obj) + "' has wrong tag!");
         } else {
            int nameAndTypeIndex = obj.getNameAndTypeIndex();
            ConstantNameAndType cnat = (ConstantNameAndType)this.cp.getConstant(nameAndTypeIndex);
            String name = ((ConstantUtf8)this.cp.getConstant(cnat.getNameIndex())).getBytes();
            if (!Pass2Verifier.validClassMethodName(name)) {
               throw new ClassConstraintException("Invalid (non-interface) method name '" + name + "' referenced by '" + Pass2Verifier.tostring(obj) + "'.");
            } else {
               int classIndex = obj.getClassIndex();
               ConstantClass cc = (ConstantClass)this.cp.getConstant(classIndex);
               String className = ((ConstantUtf8)this.cp.getConstant(cc.getNameIndex())).getBytes();
               if (!Pass2Verifier.validClassName(className)) {
                  throw new ClassConstraintException("Illegal class name '" + className + "' used by '" + Pass2Verifier.tostring(obj) + "'.");
               } else {
                  String sig = ((ConstantUtf8)this.cp.getConstant(cnat.getSignatureIndex())).getBytes();

                  try {
                     Type t = Type.getReturnType(sig);
                     if (name.equals("<init>") && t != Type.VOID) {
                        throw new ClassConstraintException("Instance initialization method must have VOID return type.");
                     }
                  } catch (ClassFormatException var10) {
                     ClassFormatException cfe = var10;
                     throw new ClassConstraintException("Illegal descriptor (==signature) '" + sig + "' used by '" + Pass2Verifier.tostring(obj) + "'.", cfe);
                  }
               }
            }
         }
      }

      // $FF: synthetic method
      FAMRAV_Visitor(JavaClass x1, Object x2) {
         this(x1);
      }
   }

   private final class CPESSC_Visitor extends EmptyVisitor {
      private final Class<?> CONST_Class;
      private final Class<?> CONST_String;
      private final Class<?> CONST_Integer;
      private final Class<?> CONST_Float;
      private final Class<?> CONST_Long;
      private final Class<?> CONST_Double;
      private final Class<?> CONST_NameAndType;
      private final Class<?> CONST_Utf8;
      private final JavaClass jc;
      private final ConstantPool cp;
      private final int cplen;
      private final DescendingVisitor carrier;
      private final Set<String> fieldNames;
      private final Set<String> fieldNamesAndDesc;
      private final Set<String> methodNamesAndDesc;

      private CPESSC_Visitor(JavaClass jc) {
         this.fieldNames = new HashSet();
         this.fieldNamesAndDesc = new HashSet();
         this.methodNamesAndDesc = new HashSet();
         this.jc = jc;
         this.cp = jc.getConstantPool();
         this.cplen = this.cp.getLength();
         this.CONST_Class = ConstantClass.class;
         this.CONST_String = ConstantString.class;
         this.CONST_Integer = ConstantInteger.class;
         this.CONST_Float = ConstantFloat.class;
         this.CONST_Long = ConstantLong.class;
         this.CONST_Double = ConstantDouble.class;
         this.CONST_NameAndType = ConstantNameAndType.class;
         this.CONST_Utf8 = ConstantUtf8.class;
         this.carrier = new DescendingVisitor(jc, this);
         this.carrier.visit();
      }

      private void checkIndex(Node referrer, int index, Class<?> shouldbe) {
         if (index >= 0 && index < this.cplen) {
            Constant c = this.cp.getConstant(index);
            if (!shouldbe.isInstance(c)) {
               throw new ClassConstraintException("Illegal constant '" + Pass2Verifier.tostring(c) + "' at index '" + index + "'. '" + Pass2Verifier.tostring(referrer) + "' expects a '" + shouldbe + "'.");
            }
         } else {
            throw new ClassConstraintException("Invalid index '" + index + "' used by '" + Pass2Verifier.tostring(referrer) + "'.");
         }
      }

      public void visitCode(Code obj) {
         try {
            this.checkIndex(obj, obj.getNameIndex(), this.CONST_Utf8);
            String name = ((ConstantUtf8)this.cp.getConstant(obj.getNameIndex())).getBytes();
            if (!name.equals("Code")) {
               throw new ClassConstraintException("The Code attribute '" + Pass2Verifier.tostring(obj) + "' is not correctly named 'Code' but '" + name + "'.");
            } else if (!(this.carrier.predecessor() instanceof Method)) {
               Pass2Verifier.this.addMessage("Code attribute '" + Pass2Verifier.tostring(obj) + "' is not declared in a method_info structure but in '" + this.carrier.predecessor() + "'. Ignored.");
            } else {
               Method m = (Method)this.carrier.predecessor();
               if (obj.getCode().length == 0) {
                  throw new ClassConstraintException("Code array of Code attribute '" + Pass2Verifier.tostring(obj) + "' (method '" + m + "') must not be empty.");
               } else {
                  CodeException[] excTable = obj.getExceptionTable();
                  CodeException[] var5 = excTable;
                  int var6 = excTable.length;

                  int numOfLvtAttribs;
                  for(numOfLvtAttribs = 0; numOfLvtAttribs < var6; ++numOfLvtAttribs) {
                     CodeException element = var5[numOfLvtAttribs];
                     int excIndex = element.getCatchType();
                     if (excIndex != 0) {
                        this.checkIndex(obj, excIndex, this.CONST_Class);
                        ConstantClass cc = (ConstantClass)this.cp.getConstant(excIndex);
                        this.checkIndex(cc, cc.getNameIndex(), this.CONST_Utf8);
                        String cname = Utility.pathToPackage(((ConstantUtf8)this.cp.getConstant(cc.getNameIndex())).getBytes());
                        Verifier v = VerifierFactory.getVerifier(cname);
                        VerificationResult vr = v.doPass1();
                        if (vr != VerificationResult.VR_OK) {
                           throw new ClassConstraintException("Code attribute '" + Pass2Verifier.tostring(obj) + "' (method '" + m + "') has an exception_table entry '" + Pass2Verifier.tostring(element) + "' that references '" + cname + "' as an Exception but it does not pass verification pass 1: " + vr);
                        }

                        JavaClass ex = Repository.lookupClass(cname);
                        JavaClass t = Repository.lookupClass(Type.THROWABLE.getClassName());

                        for(JavaClass o = Repository.lookupClass(Type.OBJECT.getClassName()); ex != o && ex != t; ex = Repository.lookupClass(ex.getSuperclassName())) {
                           v = VerifierFactory.getVerifier(ex.getSuperclassName());
                           vr = v.doPass1();
                           if (vr != VerificationResult.VR_OK) {
                              throw new ClassConstraintException("Code attribute '" + Pass2Verifier.tostring(obj) + "' (method '" + m + "') has an exception_table entry '" + Pass2Verifier.tostring(element) + "' that references '" + cname + "' as an Exception but '" + ex.getSuperclassName() + "' in the ancestor hierachy does not pass verification pass 1: " + vr);
                           }
                        }

                        if (ex != t) {
                           throw new ClassConstraintException("Code attribute '" + Pass2Verifier.tostring(obj) + "' (method '" + m + "') has an exception_table entry '" + Pass2Verifier.tostring(element) + "' that references '" + cname + "' as an Exception but it is not a subclass of '" + t.getClassName() + "'.");
                        }
                     }
                  }

                  int methodNumber = -1;
                  Method[] ms = Repository.lookupClass(Pass2Verifier.this.myOwner.getClassName()).getMethods();

                  for(numOfLvtAttribs = 0; numOfLvtAttribs < ms.length; ++numOfLvtAttribs) {
                     if (m == ms[numOfLvtAttribs]) {
                        methodNumber = numOfLvtAttribs;
                        break;
                     }
                  }

                  if (methodNumber < 0) {
                     for(numOfLvtAttribs = 0; numOfLvtAttribs < ms.length; ++numOfLvtAttribs) {
                        if (m.getName().equals(ms[numOfLvtAttribs].getName())) {
                           methodNumber = numOfLvtAttribs;
                           break;
                        }
                     }
                  }

                  if (methodNumber < 0) {
                     throw new AssertionViolatedException("Could not find a known BCEL Method object in the corresponding BCEL JavaClass object.");
                  } else {
                     Pass2Verifier.this.localVariablesInfos[methodNumber] = new LocalVariablesInfo(obj.getMaxLocals());
                     numOfLvtAttribs = 0;
                     Attribute[] atts = obj.getAttributes();
                     Attribute[] var31 = atts;
                     int var32 = atts.length;

                     for(int var33 = 0; var33 < var32; ++var33) {
                        Attribute att = var31[var33];
                        if (!(att instanceof LineNumberTable) && !(att instanceof LocalVariableTable)) {
                           Pass2Verifier.this.addMessage("Attribute '" + Pass2Verifier.tostring(att) + "' as an attribute of Code attribute '" + Pass2Verifier.tostring(obj) + "' (method '" + m + "') is unknown and will therefore be ignored.");
                        } else {
                           Pass2Verifier.this.addMessage("Attribute '" + Pass2Verifier.tostring(att) + "' as an attribute of Code attribute '" + Pass2Verifier.tostring(obj) + "' (method '" + m + "') will effectively be ignored and is only useful for debuggers and such.");
                        }

                        if (att instanceof LocalVariableTable) {
                           LocalVariableTable lvt = (LocalVariableTable)att;
                           this.checkIndex(lvt, lvt.getNameIndex(), this.CONST_Utf8);
                           String lvtname = ((ConstantUtf8)this.cp.getConstant(lvt.getNameIndex())).getBytes();
                           if (!lvtname.equals("LocalVariableTable")) {
                              throw new ClassConstraintException("The LocalVariableTable attribute '" + Pass2Verifier.tostring(lvt) + "' is not correctly named 'LocalVariableTable' but '" + lvtname + "'.");
                           }

                           LocalVariable[] var37 = lvt.getLocalVariableTable();
                           int var38 = var37.length;

                           for(int var17 = 0; var17 < var38; ++var17) {
                              LocalVariable localvariable = var37[var17];
                              this.checkIndex(lvt, localvariable.getNameIndex(), this.CONST_Utf8);
                              String localname = ((ConstantUtf8)this.cp.getConstant(localvariable.getNameIndex())).getBytes();
                              if (!Pass2Verifier.validJavaIdentifier(localname)) {
                                 throw new ClassConstraintException("LocalVariableTable '" + Pass2Verifier.tostring(lvt) + "' references a local variable by the name '" + localname + "' which is not a legal Java simple name.");
                              }

                              this.checkIndex(lvt, localvariable.getSignatureIndex(), this.CONST_Utf8);
                              String localsig = ((ConstantUtf8)this.cp.getConstant(localvariable.getSignatureIndex())).getBytes();

                              Type tx;
                              try {
                                 tx = Type.getType(localsig);
                              } catch (ClassFormatException var24) {
                                 ClassFormatException cfe = var24;
                                 throw new ClassConstraintException("Illegal descriptor (==signature) '" + localsig + "' used by LocalVariable '" + Pass2Verifier.tostring(localvariable) + "' referenced by '" + Pass2Verifier.tostring(lvt) + "'.", cfe);
                              }

                              int localindex = localvariable.getIndex();
                              if ((tx != Type.LONG && tx != Type.DOUBLE ? localindex : localindex + 1) >= obj.getMaxLocals()) {
                                 throw new ClassConstraintException("LocalVariableTable attribute '" + Pass2Verifier.tostring(lvt) + "' references a LocalVariable '" + Pass2Verifier.tostring(localvariable) + "' with an index that exceeds the surrounding Code attribute's max_locals value of '" + obj.getMaxLocals() + "'.");
                              }

                              try {
                                 Pass2Verifier.this.localVariablesInfos[methodNumber].add(localindex, localname, localvariable.getStartPC(), localvariable.getLength(), tx);
                              } catch (LocalVariableInfoInconsistentException var25) {
                                 LocalVariableInfoInconsistentException lviie = var25;
                                 throw new ClassConstraintException("Conflicting information in LocalVariableTable '" + Pass2Verifier.tostring(lvt) + "' found in Code attribute '" + Pass2Verifier.tostring(obj) + "' (method '" + Pass2Verifier.tostring(m) + "'). " + lviie.getMessage(), lviie);
                              }
                           }

                           ++numOfLvtAttribs;
                           if (!m.isStatic() && numOfLvtAttribs > obj.getMaxLocals()) {
                              throw new ClassConstraintException("Number of LocalVariableTable attributes of Code attribute '" + Pass2Verifier.tostring(obj) + "' (method '" + Pass2Verifier.tostring(m) + "') exceeds number of local variable slots '" + obj.getMaxLocals() + "' ('There may be at most one LocalVariableTable attribute per local variable in the Code attribute.').");
                           }
                        }
                     }

                  }
               }
            }
         } catch (ClassNotFoundException var26) {
            ClassNotFoundException e = var26;
            throw new AssertionViolatedException("Missing class: " + e, e);
         }
      }

      public void visitCodeException(CodeException obj) {
      }

      public void visitConstantClass(ConstantClass obj) {
         if (obj.getTag() != 7) {
            throw new ClassConstraintException("Wrong constant tag in '" + Pass2Verifier.tostring(obj) + "'.");
         } else {
            this.checkIndex(obj, obj.getNameIndex(), this.CONST_Utf8);
         }
      }

      public void visitConstantDouble(ConstantDouble obj) {
         if (obj.getTag() != 6) {
            throw new ClassConstraintException("Wrong constant tag in '" + Pass2Verifier.tostring(obj) + "'.");
         }
      }

      public void visitConstantFieldref(ConstantFieldref obj) {
         if (obj.getTag() != 9) {
            throw new ClassConstraintException("Wrong constant tag in '" + Pass2Verifier.tostring(obj) + "'.");
         } else {
            this.checkIndex(obj, obj.getClassIndex(), this.CONST_Class);
            this.checkIndex(obj, obj.getNameAndTypeIndex(), this.CONST_NameAndType);
         }
      }

      public void visitConstantFloat(ConstantFloat obj) {
         if (obj.getTag() != 4) {
            throw new ClassConstraintException("Wrong constant tag in '" + Pass2Verifier.tostring(obj) + "'.");
         }
      }

      public void visitConstantInteger(ConstantInteger obj) {
         if (obj.getTag() != 3) {
            throw new ClassConstraintException("Wrong constant tag in '" + Pass2Verifier.tostring(obj) + "'.");
         }
      }

      public void visitConstantInterfaceMethodref(ConstantInterfaceMethodref obj) {
         if (obj.getTag() != 11) {
            throw new ClassConstraintException("Wrong constant tag in '" + Pass2Verifier.tostring(obj) + "'.");
         } else {
            this.checkIndex(obj, obj.getClassIndex(), this.CONST_Class);
            this.checkIndex(obj, obj.getNameAndTypeIndex(), this.CONST_NameAndType);
         }
      }

      public void visitConstantLong(ConstantLong obj) {
         if (obj.getTag() != 5) {
            throw new ClassConstraintException("Wrong constant tag in '" + Pass2Verifier.tostring(obj) + "'.");
         }
      }

      public void visitConstantMethodref(ConstantMethodref obj) {
         if (obj.getTag() != 10) {
            throw new ClassConstraintException("Wrong constant tag in '" + Pass2Verifier.tostring(obj) + "'.");
         } else {
            this.checkIndex(obj, obj.getClassIndex(), this.CONST_Class);
            this.checkIndex(obj, obj.getNameAndTypeIndex(), this.CONST_NameAndType);
         }
      }

      public void visitConstantNameAndType(ConstantNameAndType obj) {
         if (obj.getTag() != 12) {
            throw new ClassConstraintException("Wrong constant tag in '" + Pass2Verifier.tostring(obj) + "'.");
         } else {
            this.checkIndex(obj, obj.getNameIndex(), this.CONST_Utf8);
            this.checkIndex(obj, obj.getSignatureIndex(), this.CONST_Utf8);
         }
      }

      public void visitConstantPool(ConstantPool obj) {
      }

      public void visitConstantString(ConstantString obj) {
         if (obj.getTag() != 8) {
            throw new ClassConstraintException("Wrong constant tag in '" + Pass2Verifier.tostring(obj) + "'.");
         } else {
            this.checkIndex(obj, obj.getStringIndex(), this.CONST_Utf8);
         }
      }

      public void visitConstantUtf8(ConstantUtf8 obj) {
         if (obj.getTag() != 1) {
            throw new ClassConstraintException("Wrong constant tag in '" + Pass2Verifier.tostring(obj) + "'.");
         }
      }

      public void visitConstantValue(ConstantValue obj) {
         this.checkIndex(obj, obj.getNameIndex(), this.CONST_Utf8);
         String name = ((ConstantUtf8)this.cp.getConstant(obj.getNameIndex())).getBytes();
         if (!name.equals("ConstantValue")) {
            throw new ClassConstraintException("The ConstantValue attribute '" + Pass2Verifier.tostring(obj) + "' is not correctly named 'ConstantValue' but '" + name + "'.");
         } else {
            Object pred = this.carrier.predecessor();
            if (pred instanceof Field) {
               Field f = (Field)pred;
               Type fieldType = Type.getType(((ConstantUtf8)this.cp.getConstant(f.getSignatureIndex())).getBytes());
               int index = obj.getConstantValueIndex();
               if (index >= 0 && index < this.cplen) {
                  Constant c = this.cp.getConstant(index);
                  if ((!this.CONST_Long.isInstance(c) || !fieldType.equals(Type.LONG)) && (!this.CONST_Float.isInstance(c) || !fieldType.equals(Type.FLOAT))) {
                     if (!this.CONST_Double.isInstance(c) || !fieldType.equals(Type.DOUBLE)) {
                        if (!this.CONST_Integer.isInstance(c) || !fieldType.equals(Type.INT) && !fieldType.equals(Type.SHORT) && !fieldType.equals(Type.CHAR) && !fieldType.equals(Type.BYTE) && !fieldType.equals(Type.BOOLEAN)) {
                           if (!this.CONST_String.isInstance(c) || !fieldType.equals(Type.STRING)) {
                              throw new ClassConstraintException("Illegal type of ConstantValue '" + obj + "' embedding Constant '" + c + "'. It is referenced by field '" + Pass2Verifier.tostring(f) + "' expecting a different type: '" + fieldType + "'.");
                           }
                        }
                     }
                  }
               } else {
                  throw new ClassConstraintException("Invalid index '" + index + "' used by '" + Pass2Verifier.tostring(obj) + "'.");
               }
            }
         }
      }

      public void visitDeprecated(Deprecated obj) {
         this.checkIndex(obj, obj.getNameIndex(), this.CONST_Utf8);
         String name = ((ConstantUtf8)this.cp.getConstant(obj.getNameIndex())).getBytes();
         if (!name.equals("Deprecated")) {
            throw new ClassConstraintException("The Deprecated attribute '" + Pass2Verifier.tostring(obj) + "' is not correctly named 'Deprecated' but '" + name + "'.");
         }
      }

      public void visitExceptionTable(ExceptionTable obj) {
         try {
            this.checkIndex(obj, obj.getNameIndex(), this.CONST_Utf8);
            String name = ((ConstantUtf8)this.cp.getConstant(obj.getNameIndex())).getBytes();
            if (!name.equals("Exceptions")) {
               throw new ClassConstraintException("The Exceptions attribute '" + Pass2Verifier.tostring(obj) + "' is not correctly named 'Exceptions' but '" + name + "'.");
            } else {
               int[] excIndices = obj.getExceptionIndexTable();
               int[] var4 = excIndices;
               int var5 = excIndices.length;

               for(int var6 = 0; var6 < var5; ++var6) {
                  int excIndice = var4[var6];
                  this.checkIndex(obj, excIndice, this.CONST_Class);
                  ConstantClass cc = (ConstantClass)this.cp.getConstant(excIndice);
                  this.checkIndex(cc, cc.getNameIndex(), this.CONST_Utf8);
                  String cname = Utility.pathToPackage(((ConstantUtf8)this.cp.getConstant(cc.getNameIndex())).getBytes());
                  Verifier v = VerifierFactory.getVerifier(cname);
                  VerificationResult vr = v.doPass1();
                  if (vr != VerificationResult.VR_OK) {
                     throw new ClassConstraintException("Exceptions attribute '" + Pass2Verifier.tostring(obj) + "' references '" + cname + "' as an Exception but it does not pass verification pass 1: " + vr);
                  }

                  JavaClass e = Repository.lookupClass(cname);
                  JavaClass t = Repository.lookupClass(Type.THROWABLE.getClassName());

                  for(JavaClass o = Repository.lookupClass(Type.OBJECT.getClassName()); e != o && e != t; e = Repository.lookupClass(e.getSuperclassName())) {
                     v = VerifierFactory.getVerifier(e.getSuperclassName());
                     vr = v.doPass1();
                     if (vr != VerificationResult.VR_OK) {
                        throw new ClassConstraintException("Exceptions attribute '" + Pass2Verifier.tostring(obj) + "' references '" + cname + "' as an Exception but '" + e.getSuperclassName() + "' in the ancestor hierachy does not pass verification pass 1: " + vr);
                     }
                  }

                  if (e != t) {
                     throw new ClassConstraintException("Exceptions attribute '" + Pass2Verifier.tostring(obj) + "' references '" + cname + "' as an Exception but it is not a subclass of '" + t.getClassName() + "'.");
                  }
               }

            }
         } catch (ClassNotFoundException var15) {
            ClassNotFoundException ex = var15;
            throw new AssertionViolatedException("Missing class: " + ex, ex);
         }
      }

      public void visitField(Field obj) {
         if (this.jc.isClass()) {
            int maxone = 0;
            if (obj.isPrivate()) {
               ++maxone;
            }

            if (obj.isProtected()) {
               ++maxone;
            }

            if (obj.isPublic()) {
               ++maxone;
            }

            if (maxone > 1) {
               throw new ClassConstraintException("Field '" + Pass2Verifier.tostring(obj) + "' must only have at most one of its ACC_PRIVATE, ACC_PROTECTED, ACC_PUBLIC modifiers set.");
            }

            if (obj.isFinal() && obj.isVolatile()) {
               throw new ClassConstraintException("Field '" + Pass2Verifier.tostring(obj) + "' must only have at most one of its ACC_FINAL, ACC_VOLATILE modifiers set.");
            }
         } else {
            if (!obj.isPublic()) {
               throw new ClassConstraintException("Interface field '" + Pass2Verifier.tostring(obj) + "' must have the ACC_PUBLIC modifier set but hasn't!");
            }

            if (!obj.isStatic()) {
               throw new ClassConstraintException("Interface field '" + Pass2Verifier.tostring(obj) + "' must have the ACC_STATIC modifier set but hasn't!");
            }

            if (!obj.isFinal()) {
               throw new ClassConstraintException("Interface field '" + Pass2Verifier.tostring(obj) + "' must have the ACC_FINAL modifier set but hasn't!");
            }
         }

         if ((obj.getAccessFlags() & -224) > 0) {
            Pass2Verifier.this.addMessage("Field '" + Pass2Verifier.tostring(obj) + "' has access flag(s) other than ACC_PUBLIC, ACC_PRIVATE, ACC_PROTECTED, ACC_STATIC, ACC_FINAL, ACC_VOLATILE, ACC_TRANSIENT set (ignored).");
         }

         this.checkIndex(obj, obj.getNameIndex(), this.CONST_Utf8);
         String name = obj.getName();
         if (!Pass2Verifier.validFieldName(name)) {
            throw new ClassConstraintException("Field '" + Pass2Verifier.tostring(obj) + "' has illegal name '" + obj.getName() + "'.");
         } else {
            this.checkIndex(obj, obj.getSignatureIndex(), this.CONST_Utf8);
            String sig = ((ConstantUtf8)this.cp.getConstant(obj.getSignatureIndex())).getBytes();

            try {
               Type.getType(sig);
            } catch (ClassFormatException var10) {
               ClassFormatException cfe = var10;
               throw new ClassConstraintException("Illegal descriptor (==signature) '" + sig + "' used by '" + Pass2Verifier.tostring(obj) + "'.", cfe);
            }

            String nameanddesc = name + sig;
            if (this.fieldNamesAndDesc.contains(nameanddesc)) {
               throw new ClassConstraintException("No two fields (like '" + Pass2Verifier.tostring(obj) + "') are allowed have same names and descriptors!");
            } else {
               if (this.fieldNames.contains(name)) {
                  Pass2Verifier.this.addMessage("More than one field of name '" + name + "' detected (but with different type descriptors). This is very unusual.");
               }

               this.fieldNamesAndDesc.add(nameanddesc);
               this.fieldNames.add(name);
               Attribute[] atts = obj.getAttributes();
               Attribute[] var6 = atts;
               int var7 = atts.length;

               for(int var8 = 0; var8 < var7; ++var8) {
                  Attribute att = var6[var8];
                  if (!(att instanceof ConstantValue) && !(att instanceof Synthetic) && !(att instanceof Deprecated)) {
                     Pass2Verifier.this.addMessage("Attribute '" + Pass2Verifier.tostring(att) + "' as an attribute of Field '" + Pass2Verifier.tostring(obj) + "' is unknown and will therefore be ignored.");
                  }

                  if (!(att instanceof ConstantValue)) {
                     Pass2Verifier.this.addMessage("Attribute '" + Pass2Verifier.tostring(att) + "' as an attribute of Field '" + Pass2Verifier.tostring(obj) + "' is not a ConstantValue and is therefore only of use for debuggers and such.");
                  }
               }

            }
         }
      }

      public void visitInnerClass(InnerClass obj) {
      }

      public void visitInnerClasses(InnerClasses innerClasses) {
         this.checkIndex(innerClasses, innerClasses.getNameIndex(), this.CONST_Utf8);
         String name = ((ConstantUtf8)this.cp.getConstant(innerClasses.getNameIndex())).getBytes();
         if (!name.equals("InnerClasses")) {
            throw new ClassConstraintException("The InnerClasses attribute '" + Pass2Verifier.tostring(innerClasses) + "' is not correctly named 'InnerClasses' but '" + name + "'.");
         } else {
            innerClasses.forEach((ic) -> {
               this.checkIndex(innerClasses, ic.getInnerClassIndex(), this.CONST_Class);
               int outerIdx = ic.getOuterClassIndex();
               if (outerIdx != 0) {
                  this.checkIndex(innerClasses, outerIdx, this.CONST_Class);
               }

               int innernameIdx = ic.getInnerNameIndex();
               if (innernameIdx != 0) {
                  this.checkIndex(innerClasses, innernameIdx, this.CONST_Utf8);
               }

               int acc = ic.getInnerAccessFlags();
               acc &= -1568;
               if (acc != 0) {
                  Pass2Verifier.this.addMessage("Unknown access flag for inner class '" + Pass2Verifier.tostring(ic) + "' set (InnerClasses attribute '" + Pass2Verifier.tostring(innerClasses) + "').");
               }

            });
         }
      }

      public void visitJavaClass(JavaClass obj) {
         Attribute[] atts = obj.getAttributes();
         boolean foundSourceFile = false;
         boolean foundInnerClasses = false;
         boolean hasInnerClass = (new InnerClassDetector(this.jc)).innerClassReferenced();
         Attribute[] var6 = atts;
         int var7 = atts.length;

         for(int var8 = 0; var8 < var7; ++var8) {
            Attribute att = var6[var8];
            if (!(att instanceof SourceFile) && !(att instanceof Deprecated) && !(att instanceof InnerClasses) && !(att instanceof Synthetic)) {
               Pass2Verifier.this.addMessage("Attribute '" + Pass2Verifier.tostring(att) + "' as an attribute of the ClassFile structure '" + Pass2Verifier.tostring(obj) + "' is unknown and will therefore be ignored.");
            }

            if (att instanceof SourceFile) {
               if (foundSourceFile) {
                  throw new ClassConstraintException("A ClassFile structure (like '" + Pass2Verifier.tostring(obj) + "') may have no more than one SourceFile attribute.");
               }

               foundSourceFile = true;
            }

            if (att instanceof InnerClasses) {
               if (!foundInnerClasses) {
                  foundInnerClasses = true;
               } else if (hasInnerClass) {
                  throw new ClassConstraintException("A Classfile structure (like '" + Pass2Verifier.tostring(obj) + "') must have exactly one InnerClasses attribute if at least one Inner Class is referenced (which is the case). More than one InnerClasses attribute was found.");
               }

               if (!hasInnerClass) {
                  Pass2Verifier.this.addMessage("No referenced Inner Class found, but InnerClasses attribute '" + Pass2Verifier.tostring(att) + "' found. Strongly suggest removal of that attribute.");
               }
            }
         }

         if (hasInnerClass && !foundInnerClasses) {
            Pass2Verifier.this.addMessage("A Classfile structure (like '" + Pass2Verifier.tostring(obj) + "') must have exactly one InnerClasses attribute if at least one Inner Class is referenced (which is the case). No InnerClasses attribute was found.");
         }

      }

      public void visitLineNumber(LineNumber obj) {
      }

      public void visitLineNumberTable(LineNumberTable obj) {
         this.checkIndex(obj, obj.getNameIndex(), this.CONST_Utf8);
         String name = ((ConstantUtf8)this.cp.getConstant(obj.getNameIndex())).getBytes();
         if (!name.equals("LineNumberTable")) {
            throw new ClassConstraintException("The LineNumberTable attribute '" + Pass2Verifier.tostring(obj) + "' is not correctly named 'LineNumberTable' but '" + name + "'.");
         }
      }

      public void visitLocalVariable(LocalVariable obj) {
      }

      public void visitLocalVariableTable(LocalVariableTable obj) {
      }

      public void visitMethod(Method obj) {
         this.checkIndex(obj, obj.getNameIndex(), this.CONST_Utf8);
         String name = obj.getName();
         if (!Pass2Verifier.validMethodName(name, true)) {
            throw new ClassConstraintException("Method '" + Pass2Verifier.tostring(obj) + "' has illegal name '" + name + "'.");
         } else {
            this.checkIndex(obj, obj.getSignatureIndex(), this.CONST_Utf8);
            String sig = ((ConstantUtf8)this.cp.getConstant(obj.getSignatureIndex())).getBytes();

            Type t;
            Type[] ts;
            try {
               t = Type.getReturnType(sig);
               ts = Type.getArgumentTypes(sig);
            } catch (ClassFormatException var14) {
               ClassFormatException cfe = var14;
               throw new ClassConstraintException("Illegal descriptor (==signature) '" + sig + "' used by Method '" + Pass2Verifier.tostring(obj) + "'.", cfe);
            }

            Type act = t;
            if (t instanceof ArrayType) {
               act = ((ArrayType)t).getBasicType();
            }

            if (act instanceof ObjectType) {
               Verifier vx = VerifierFactory.getVerifier(((ObjectType)act).getClassName());
               VerificationResult vrx = vx.doPass1();
               if (vrx != VerificationResult.VR_OK) {
                  throw new ClassConstraintException("Method '" + Pass2Verifier.tostring(obj) + "' has a return type that does not pass verification pass 1: '" + vrx + "'.");
               }
            }

            Type[] var16 = ts;
            int var18 = ts.length;

            int numCodeAtts;
            for(numCodeAtts = 0; numCodeAtts < var18; ++numCodeAtts) {
               Type element = var16[numCodeAtts];
               act = element;
               if (element instanceof ArrayType) {
                  act = ((ArrayType)element).getBasicType();
               }

               if (act instanceof ObjectType) {
                  Verifier v = VerifierFactory.getVerifier(((ObjectType)act).getClassName());
                  VerificationResult vr = v.doPass1();
                  if (vr != VerificationResult.VR_OK) {
                     throw new ClassConstraintException("Method '" + Pass2Verifier.tostring(obj) + "' has an argument type that does not pass verification pass 1: '" + vr + "'.");
                  }
               }
            }

            if (name.equals("<clinit>") && ts.length != 0) {
               throw new ClassConstraintException("Method '" + Pass2Verifier.tostring(obj) + "' has illegal name '" + name + "'. Its name resembles the class or interface initialization method which it isn't because of its arguments (==descriptor).");
            } else {
               if (this.jc.isClass()) {
                  int maxone = 0;
                  if (obj.isPrivate()) {
                     ++maxone;
                  }

                  if (obj.isProtected()) {
                     ++maxone;
                  }

                  if (obj.isPublic()) {
                     ++maxone;
                  }

                  if (maxone > 1) {
                     throw new ClassConstraintException("Method '" + Pass2Verifier.tostring(obj) + "' must only have at most one of its ACC_PRIVATE, ACC_PROTECTED, ACC_PUBLIC modifiers set.");
                  }

                  if (obj.isAbstract()) {
                     if (obj.isFinal()) {
                        throw new ClassConstraintException("Abstract method '" + Pass2Verifier.tostring(obj) + "' must not have the ACC_FINAL modifier set.");
                     }

                     if (obj.isNative()) {
                        throw new ClassConstraintException("Abstract method '" + Pass2Verifier.tostring(obj) + "' must not have the ACC_NATIVE modifier set.");
                     }

                     if (obj.isPrivate()) {
                        throw new ClassConstraintException("Abstract method '" + Pass2Verifier.tostring(obj) + "' must not have the ACC_PRIVATE modifier set.");
                     }

                     if (obj.isStatic()) {
                        throw new ClassConstraintException("Abstract method '" + Pass2Verifier.tostring(obj) + "' must not have the ACC_STATIC modifier set.");
                     }

                     if (obj.isStrictfp()) {
                        throw new ClassConstraintException("Abstract method '" + Pass2Verifier.tostring(obj) + "' must not have the ACC_STRICT modifier set.");
                     }

                     if (obj.isSynchronized()) {
                        throw new ClassConstraintException("Abstract method '" + Pass2Verifier.tostring(obj) + "' must not have the ACC_SYNCHRONIZED modifier set.");
                     }
                  }

                  if (name.equals("<init>") && (obj.isStatic() || obj.isFinal() || obj.isSynchronized() || obj.isNative() || obj.isAbstract())) {
                     throw new ClassConstraintException("Instance initialization method '" + Pass2Verifier.tostring(obj) + "' must not have any of the ACC_STATIC, ACC_FINAL, ACC_SYNCHRONIZED, ACC_NATIVE, ACC_ABSTRACT modifiers set.");
                  }
               } else if (!name.equals("<clinit>")) {
                  if (this.jc.getMajor() >= 52) {
                     if (obj.isPublic() == obj.isPrivate()) {
                        throw new ClassConstraintException("Interface method '" + Pass2Verifier.tostring(obj) + "' must have exactly one of its ACC_PUBLIC and ACC_PRIVATE modifiers set.");
                     }

                     if (obj.isProtected() || obj.isFinal() || obj.isSynchronized() || obj.isNative()) {
                        throw new ClassConstraintException("Interface method '" + Pass2Verifier.tostring(obj) + "' must not have any of the ACC_PROTECTED, ACC_FINAL, ACC_SYNCHRONIZED, or ACC_NATIVE modifiers set.");
                     }
                  } else {
                     if (!obj.isPublic()) {
                        throw new ClassConstraintException("Interface method '" + Pass2Verifier.tostring(obj) + "' must have the ACC_PUBLIC modifier set but hasn't!");
                     }

                     if (!obj.isAbstract()) {
                        throw new ClassConstraintException("Interface method '" + Pass2Verifier.tostring(obj) + "' must have the ACC_ABSTRACT modifier set but hasn't!");
                     }

                     if (obj.isPrivate() || obj.isProtected() || obj.isStatic() || obj.isFinal() || obj.isSynchronized() || obj.isNative() || obj.isStrictfp()) {
                        throw new ClassConstraintException("Interface method '" + Pass2Verifier.tostring(obj) + "' must not have any of the ACC_PRIVATE, ACC_PROTECTED, ACC_STATIC, ACC_FINAL, ACC_SYNCHRONIZED, ACC_NATIVE, ACC_ABSTRACT, ACC_STRICT modifiers set.");
                     }
                  }
               }

               if ((obj.getAccessFlags() & -3392) > 0) {
                  Pass2Verifier.this.addMessage("Method '" + Pass2Verifier.tostring(obj) + "' has access flag(s) other than ACC_PUBLIC, ACC_PRIVATE, ACC_PROTECTED, ACC_STATIC, ACC_FINAL, ACC_SYNCHRONIZED, ACC_NATIVE, ACC_ABSTRACT, ACC_STRICT set (ignored).");
               }

               String nameanddesc = name + sig;
               if (this.methodNamesAndDesc.contains(nameanddesc)) {
                  throw new ClassConstraintException("No two methods (like '" + Pass2Verifier.tostring(obj) + "') are allowed have same names and desciptors!");
               } else {
                  this.methodNamesAndDesc.add(nameanddesc);
                  Attribute[] atts = obj.getAttributes();
                  numCodeAtts = 0;
                  Attribute[] var21 = atts;
                  int var22 = atts.length;

                  for(int var23 = 0; var23 < var22; ++var23) {
                     Attribute att = var21[var23];
                     if (!(att instanceof Code) && !(att instanceof ExceptionTable) && !(att instanceof Synthetic) && !(att instanceof Deprecated)) {
                        Pass2Verifier.this.addMessage("Attribute '" + Pass2Verifier.tostring(att) + "' as an attribute of Method '" + Pass2Verifier.tostring(obj) + "' is unknown and will therefore be ignored.");
                     }

                     if (!(att instanceof Code) && !(att instanceof ExceptionTable)) {
                        Pass2Verifier.this.addMessage("Attribute '" + Pass2Verifier.tostring(att) + "' as an attribute of Method '" + Pass2Verifier.tostring(obj) + "' is neither Code nor Exceptions and is therefore only of use for debuggers and such.");
                     }

                     if (att instanceof Code && (obj.isNative() || obj.isAbstract())) {
                        throw new ClassConstraintException("Native or abstract methods like '" + Pass2Verifier.tostring(obj) + "' must not have a Code attribute like '" + Pass2Verifier.tostring(att) + "'.");
                     }

                     if (att instanceof Code) {
                        ++numCodeAtts;
                     }
                  }

                  if (!obj.isNative() && !obj.isAbstract() && numCodeAtts != 1) {
                     throw new ClassConstraintException("Non-native, non-abstract methods like '" + Pass2Verifier.tostring(obj) + "' must have exactly one Code attribute (found: " + numCodeAtts + ").");
                  }
               }
            }
         }
      }

      public void visitSourceFile(SourceFile obj) {
         this.checkIndex(obj, obj.getNameIndex(), this.CONST_Utf8);
         String name = ((ConstantUtf8)this.cp.getConstant(obj.getNameIndex())).getBytes();
         if (!name.equals("SourceFile")) {
            throw new ClassConstraintException("The SourceFile attribute '" + Pass2Verifier.tostring(obj) + "' is not correctly named 'SourceFile' but '" + name + "'.");
         } else {
            this.checkIndex(obj, obj.getSourceFileIndex(), this.CONST_Utf8);
            String sourceFileName = ((ConstantUtf8)this.cp.getConstant(obj.getSourceFileIndex())).getBytes();
            String sourceFileNameLc = sourceFileName.toLowerCase(Locale.ENGLISH);
            if (sourceFileName.indexOf(47) != -1 || sourceFileName.indexOf(92) != -1 || sourceFileName.indexOf(58) != -1 || sourceFileNameLc.lastIndexOf(".java") == -1) {
               Pass2Verifier.this.addMessage("SourceFile attribute '" + Pass2Verifier.tostring(obj) + "' has a funny name: remember not to confuse certain parsers working on javap's output. Also, this name ('" + sourceFileName + "') is considered an unqualified (simple) file name only.");
            }

         }
      }

      public void visitSynthetic(Synthetic obj) {
         this.checkIndex(obj, obj.getNameIndex(), this.CONST_Utf8);
         String name = ((ConstantUtf8)this.cp.getConstant(obj.getNameIndex())).getBytes();
         if (!name.equals("Synthetic")) {
            throw new ClassConstraintException("The Synthetic attribute '" + Pass2Verifier.tostring(obj) + "' is not correctly named 'Synthetic' but '" + name + "'.");
         }
      }

      public void visitUnknown(Unknown obj) {
         this.checkIndex(obj, obj.getNameIndex(), this.CONST_Utf8);
         Pass2Verifier.this.addMessage("Unknown attribute '" + Pass2Verifier.tostring(obj) + "'. This attribute is not known in any context!");
      }

      // $FF: synthetic method
      CPESSC_Visitor(JavaClass x1, Object x2) {
         this(x1);
      }
   }
}
